/*
    <one line to give the program's name and a brief idea of what it does.>
    Copyright (C) 2010  Rico Rommel

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "configdialog.h"
#include <QMessageBox>
#include <QDebug>
#include <QSqlError>
#include <QSortFilterProxyModel>

ConfigDialog::ConfigDialog(QWidget* parent): KDialog(parent)
{
    QWidget *widget = new QWidget( this );
    ui.setupUi(widget);
    setMainWidget( widget );

    

    connect(ui.DriverBox,SIGNAL(currentIndexChanged(QString)), this, SLOT(driverChanged(QString)));
    connect(ui.ConnectButton,SIGNAL(clicked()), this, SLOT(connectButtonclicked()));
    connect(ui.TableComboBox,SIGNAL(currentIndexChanged(QString)),this,SLOT(tableChanged(QString)));
    connect(ui.CreateTableButton,SIGNAL(clicked(bool)),this, SLOT(createTable()));

}

ConfigDialog::~ConfigDialog()
{
    db.close();
}

void ConfigDialog::driverChanged(QString driver)
{
  Q_UNUSED(driver);
}


void ConfigDialog::setHostname(QString hostname)
{
    ui.ServerEdit->setText(hostname);
}

QString ConfigDialog::hostname()
{
    return ui.ServerEdit->text();
}

void ConfigDialog::setDatabase(QString dbname)
{
    ui.DatabaseEdit->setText(dbname);
}


QString ConfigDialog::database()
{
    return ui.DatabaseEdit->text();
}


void ConfigDialog::setDriver(QString driver)
{
    int index;
    index = ui.DriverBox->findText(driver);
    if (index >= 0) ui.DriverBox->setCurrentIndex(index);
    else {
        ui.DriverBox->addItem(driver);
        ui.DriverBox->setCurrentIndex(ui.DriverBox->count()-1);
    }
}

QString ConfigDialog::driver()
{
    return ui.DriverBox->currentText();
}

void ConfigDialog::setPassword(QString password)
{
    ui.PasswordEdit->setText(password);
}

QString ConfigDialog::password()
{
    return ui.PasswordEdit->text();
}

void ConfigDialog::setPort(int port)
{
    ui.PortSpinBox->setValue(port);
}


int ConfigDialog::port()
{
    return ui.PortSpinBox->value();
}

void ConfigDialog::setSavePassword(bool savepass)
{
    ui.KWalletCheckBox->setChecked(savepass);
}

bool ConfigDialog::savePassword()
{
    return ui.KWalletCheckBox->checkState();
}

void ConfigDialog::setUsername(QString username)
{
    ui.UserEdit->setText(username);
}

QString ConfigDialog::username()
{
    return ui.UserEdit->text();
}

void ConfigDialog::setTable(QString table)
{
    ui.TableComboBox->clear();
    ui.TableComboBox->addItem(table);

}

QString ConfigDialog::table()
{
    return ui.TableComboBox->currentText();
}

bool ConfigDialog::readOnly()
{
  return ui.ReadOnlyCheckBox->isChecked();
  
}

void ConfigDialog::setReadOnly(bool mode)
{
  ui.ReadOnlyCheckBox->setChecked(mode);
}


void ConfigDialog::connectButtonclicked()
{
    openDataBase();
    if (db.isOpen()) {
        ui.TableComboBox->clear();
        ui.TableComboBox->addItems(db.tables());
    }
    else QMessageBox::critical(this,"Could not open database",db.lastError().text());
}

void ConfigDialog::tableChanged(QString table)
{
  if (!db.isOpen()) openDataBase();
    if (db.isOpen()) {
        QSqlQuery query("SELECT COLUMN_NAME FROM INFORMATION_SCHEMA.STATISTICS WHERE INDEX_NAME='PRIMARY' AND INDEX_SCHEMA='" + ui.DatabaseEdit->text() + "' AND TABLE_NAME='" + table + "'", db);
        query.first () ;
        QString primary = query.value(0).toString();

        QSqlRecord rec = db.record(table);

        int n;
	//Destroy widgets in lists
	for ( n = 0; n <DatabaseFieldsList.count(); n++) {
	  DatabaseFieldsList.at(n)->setAttribute(Qt::WA_DeleteOnClose, true);
	  DatabaseFieldsList.at(n)->close();
	  KabcFieldsList.at(n)->setAttribute(Qt::WA_DeleteOnClose, true);
	  KabcFieldsList.at(n)->close();
	  
    }

        QLabel *fieldlabel;
        QComboBox *kabccombobox;
        DatabaseFieldsList.clear();
        KabcFieldsList.clear();
        for (n = 0; n < rec.count(); n++)
            if (rec.fieldName(n) != primary ) {
                fieldlabel = new QLabel(rec.fieldName(n),ui.MapWidget);
                fieldlabel->setFrameShape(QFrame::Box);
                fieldlabel->setAlignment(Qt::AlignRight);
                ui.MapLayout->addWidget(fieldlabel,n,0);
                DatabaseFieldsList.append( fieldlabel );
		
                kabccombobox = new QComboBox(ui.MapWidget);
                kabccombobox->addItems(KabcFieldsNameList);
                connect(kabccombobox,SIGNAL(currentIndexChanged(int)),SLOT(kabcIndexChanged(int)));
                ui.MapLayout->addWidget(kabccombobox,n,1);
                KabcFieldsList.append(kabccombobox);

            }


    }

}

void ConfigDialog::openDataBase()
{
    if (db.isOpen()) db.close();
    db = QSqlDatabase::addDatabase(ui.DriverBox->currentText(), "config" );
    db.setHostName(ui.ServerEdit->text());
    db.setDatabaseName(ui.DatabaseEdit->text());
    db.open(ui.UserEdit->text(), ui.PasswordEdit->text());
    if (db.isOpen()) ui.CreateTableButton->setEnabled(true);

}

QMap< QString, AddresseeTypes::Types > ConfigDialog::map()
{
    int n;
    QMap <QString, AddresseeTypes::Types> map;
    for (n = 0; n < DatabaseFieldsList.count(); n++) {
      map.insert(DatabaseFieldsList.at(n)->text(), (AddresseeTypes::Types)KabcFieldsList.at(n)->currentIndex());
    }
    return map;
}

void ConfigDialog::setMap(QMap< QString, AddresseeTypes::Types > map)
{
  int n;
  for ( n = 0 ; n < DatabaseFieldsList.count(); n++ ) {
    KabcFieldsList.at(n)->setCurrentIndex((int)map.value(DatabaseFieldsList.at(n)->text())); 
  }

    
}


void ConfigDialog::createTable()
{
    InputDialog Inputdlg("database name:", this) ;
    if (Inputdlg.exec() == KDialog::Accepted && db.isOpen()) {
        QString table = Inputdlg.input();
        if (db.tables().indexOf(table) < 0) {
            QSqlQuery query("CREATE TABLE `" + table + "` (`id` INT NOT NULL AUTO_INCREMENT, PRIMARY KEY(`id`)) ROW_FORMAT = DYNAMIC", db);
            qDebug() << query.lastQuery();
            qDebug() << query.lastError().text();
            ui.TableComboBox->clear();
            ui.TableComboBox->addItems(db.tables());
            ui.TableComboBox->setCurrentIndex(ui.TableComboBox->findText(table));

        }
        else QMessageBox::critical(this,"error creating table","table exists");

    }

}


bool ConfigDialog::autogenerate()
{
    return ui.AutoCheckBox->isChecked();
}

void ConfigDialog::setAutogenerate(bool gen)
{
    ui.AutoCheckBox->setChecked(gen);
}

void ConfigDialog::setMultipleKabcFields(QList<AddresseeTypes::Types> fields)
{
    MultipleFields = fields;

}

void ConfigDialog::kabcIndexChanged(int index)
{
  Q_UNUSED( index );
  int n = 0;
  int m = 0;
  bool conflict = false;
  while ( n < KabcFieldsList.count() -1  && !conflict) {
    m = n + 1;
    while ( m < KabcFieldsList.count() && !conflict) {
      if (KabcFieldsList.at(n)->currentIndex() == KabcFieldsList.at(m)->currentIndex() )
	if (!MultipleFields.contains( (AddresseeTypes::Types) KabcFieldsList.at(n)->currentIndex() )) conflict = true;
      m++;
    }
    n++;
  }
  QPalette pal = ui.ErrorLabel->palette();
  QColor col;
  if (conflict) {
    col = Qt::red;
    ui.ErrorLabel->setText(i18n("Conflict between") + " " + DatabaseFieldsList.at(n-1)->text() + " " + i18n("and") + " " + DatabaseFieldsList.at(m-1)->text());
    enableButtonOk(false);
  }
  else {
      ui.ErrorLabel->setBackgroundRole(backgroundRole());
      ui.ErrorLabel->setText("");
      col = Qt::green;
      enableButtonOk(true);
  }
  pal.setColor(ui.ErrorLabel->backgroundRole(), col);
  ui.ErrorLabel->setPalette(pal);
}




void ConfigDialog::setTypeAttributesMap (QMap< AddresseeTypes::Types, struct typeattributes > map)
{
  int n;
  TypeAttributesMap = map;
  QList<struct typeattributes> vlist;
  vlist = TypeAttributesMap.values();
  for ( n = 0 ; n < vlist.count(); n++) 
  KabcFieldsNameList << vlist.at(n).Text;
}

QString ConfigDialog::resourceName()
{
  return ui.ResourceNameEdit->text();
}

void ConfigDialog::setResourceName(QString resourcename)
{
  ui.ResourceNameEdit->setText(resourcename);
}

#include "configdialog.moc"
